from django.core.exceptions import ValidationError

from arkindex.dataimport.models import WorkerVersion
from arkindex.documents.models import Corpus, Entity, EntityLink, EntityRole, EntityType, MetaData, MetaType
from arkindex.project.tests import FixtureTestCase


class TestSaveEntities(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        worker_version = WorkerVersion.objects.first()
        cls.corpus1 = Corpus.objects.create(name='corpus 1')
        cls.corpus2 = Corpus.objects.create(name='corpus 2')
        cls.parent = Entity.objects.create(
            name='parent',
            type=EntityType.Organization,
            corpus=cls.corpus1,
            worker_version=worker_version,
        )
        cls.child = Entity.objects.create(
            type=EntityType.Person,
            corpus=cls.corpus1,
            name="child",
            worker_version=worker_version,
        )
        cls.role = EntityRole.objects.create(
            parent_name='organization',
            child_name='person',
            parent_type=EntityType.Organization,
            child_type=EntityType.Person,
            corpus=cls.corpus1,
        )
        cls.link = EntityLink(parent=cls.parent, child=cls.child, role=cls.role)

    def test_parent_type_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Person
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_parent_corpus_different(self):
        self.parent.corpus = self.corpus2
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_child_type_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Organization
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_child_corpus_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus2
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_save_entity_in_metadata(self):
        self.parent.corpus = self.corpus2
        element = self.corpus1.elements.create(
            type=self.corpus1.types.create(slug='act'),
            name="element",
        )
        with self.assertRaises(ValidationError):
            MetaData.objects.create(
                name='test 1',
                type=MetaType.Text,
                value='Blah',
                element=element,
                entity=self.parent,
            )
