from django.test import TestCase

from arkindex.documents.date_parser import parse_date
from arkindex.documents.dates import DateType, InterpretedDate


class TestInterpretedDate(TestCase):

    def test_date_str(self):
        interpreted_date = parse_date('1337-may')
        self.assertEqual(1, len(interpreted_date))
        self.assertEqual('1337-05', str(interpreted_date[0]))

    def test_to_es_range(self):
        date_test_table = {
            InterpretedDate(
                year=1221,
                type=DateType.Exact,
            ): {'gte': '1221', 'lt': '1221||+1y'},
            InterpretedDate(
                year=1350,
                month=2,
                type=DateType.Exact,
            ): {'gte': '1350-02', 'lt': '1350-02||+1M'},
            InterpretedDate(
                year=1323,
                type=DateType.Lower,
            ): {'gte': '1323'},
            InterpretedDate(
                year=1212,
                month=12,
                day=12,
                type=DateType.Lower,
            ): {'gte': '1212-12-12'},
            InterpretedDate(
                year=1212,
                type=DateType.Upper,
            ): {'lt': '1212||+1y'},
            InterpretedDate(
                year=700,
                type=DateType.Unknown,
            ): {},
        }
        for interpreted_date, expected_range in date_test_table.items():
            self.assertEqual(interpreted_date.to_es_range(), expected_range)
